@@ -4,7 +4,7 @@ from flask import Blueprint, current_app, g, jsonify, redirect, request
 from rq import Connection, Queue
 
 from .build import build
-from .common import get_request_hash
+from .common import get_request_hash, stats_versions, stats_profiles
 
 bp = Blueprint("api", __name__, url_prefix="/api")
 
@@ -29,10 +29,26 @@ def get_redis():
     return g.redis
 
 
-# legacy
-@bp.route("/branches")
-def api_branches():
-    return redirect("/json/v1/branches.json")
+def get_queue() -> Queue:
+    """Return the current queue
+
+    Returns:
+        Queue: The current RQ work queue
+    """
+    if "queue" not in g:
+        with Connection():
+            g.queue = Queue(connection=get_redis())
+    return g.queue
+
+
+def api_v1_revision(version, target, subtarget):
+    return jsonify(
+        {
+            "revision": get_redis()
+            .get(f"revision-{version}-{target}/{subtarget}" or b"")
+            .decode()
+        }
+    )
 
 
 # tbd
@@ -41,6 +57,21 @@ def api_latest():
     return redirect("/json/v1/latest.json")
 
 
+def api_v1_stats_images():
+    return jsonify(
+        {
+            "total": int((get_redis().get("stats-images") or b"0").decode("utf-8")),
+            "custom": int(
+                (get_redis().get("stats-images-custom") or b"0").decode("utf-8")
+            ),
+        }
+    )
+
+
+def api_v1_stats_versions():
+    return jsonify({"versions": stats_versions()})
+
+
 def api_v1_stats_targets(branch="SNAPSHOT"):
     if branch not in current_app.config["BRANCHES"]:
         return "", 404
@@ -50,7 +81,7 @@ def api_v1_stats_targets(branch="SNAPSHOT"):
             "branch": branch,
             "targets": [
                 (s, p.decode("utf-8"))
-                for p, s in get_redis().zrange(
+                for p, s in get_redis().zrevrange(
                     f"stats-targets-{branch}", 0, -1, withscores=True
                 )
             ],
@@ -72,7 +103,7 @@ def api_v1_stats_packages(branch="SNAPSHOT"):
             "branch": branch,
             "packages": [
                 (s, p.decode("utf-8"))
-                for p, s in get_redis().zrange(
+                for p, s in get_redis().zrevrange(
                     f"stats-packages-{branch}", 0, -1, withscores=True
                 )
             ],
@@ -89,17 +120,7 @@ def api_v1_stats_profiles(branch):
     if branch not in current_app.config["BRANCHES"]:
         return "", 404
 
-    return jsonify(
-        {
-            "branch": branch,
-            "profiles": [
-                (s, p.decode("utf-8"))
-                for p, s in get_redis().zrange(
-                    f"stats-profiles-{branch}", 0, -1, withscores=True
-                )
-            ],
-        }
-    )
+    return jsonify({"branch": branch, "profiles": stats_profiles(branch)})
 
 
 @bp.route("/v1/stats/profiles/")
@@ -107,18 +128,6 @@ def api_v1_stats_profiles_default():
     return redirect("/api/v1/stats/profiles/SNAPSHOT")
 
 
-def get_queue() -> Queue:
-    """Return the current queue
-
-    Returns:
-        Queue: The current RQ work queue
-    """
-    if "queue" not in g:
-        with Connection():
-            g.queue = Queue(connection=get_redis())
-    return g.queue
-
-
 def validate_packages(req):
     if req.get("packages_versions") and not req.get("packages"):
         req["packages"] = req["packages_versions"].keys()
@@ -176,6 +185,13 @@ def validate_request(req):
         (dict, int): Status message and code, empty if no error appears
 
     """
+
+    if "defaults" in req and not current_app.config["ALLOW_DEFAULTS"]:
+        return (
+            {"detail": f"Handling `defaults` not enabled on server", "status": 400},
+            400,
+        )
+
     req["distro"] = req.get("distro", "openwrt")
     if req["distro"] not in get_distros():
         return (
@@ -183,8 +199,6 @@ def validate_request(req):
             400,
         )
 
-    req["version"] = req["version"]
-
     if req["version"].endswith("-SNAPSHOT"):
         # e.g. 21.02-snapshot
         req["branch"] = req["version"].rsplit("-", maxsplit=1)[0]
@@ -217,33 +231,33 @@ def validate_request(req):
             400,
         )
 
-    req["arch"] = current_app.config["BRANCHES"][req["branch"]]["targets"][
-        req["target"]
-    ]
+    req["arch"] = (
+        r.hget(f"architecture-{req['branch']}", req["target"]) or b""
+    ).decode()
 
     if req["target"] in ["x86/64", "x86/generic", "x86/geode", "x86/legacy"]:
         current_app.logger.debug("Use generic profile for {req['target']}")
         req["profile"] = "generic"
     else:
-        mapped_profile = r.hget(
-            f"mapping-{req['branch']}-{req['version']}-{req['target']}",
-            req["profile"],
-        )
-
-        if mapped_profile:
-            req["profile"] = mapped_profile.decode()
-
-        current_app.logger.debug("Profile after mapping " + req["profile"])
-
-        if not r.sismember(
+        if r.sismember(
             f"profiles-{req['branch']}-{req['version']}-{req['target']}",
-            req["profile"],
+            req["profile"].replace(",", "_"),
         ):
-            return (
-                {"detail": f"Unsupported profile: {req['profile']}", "status": 400},
-                400,
+            req["profile"] = req["profile"].replace(",", "_")
+        else:
+            mapped_profile = r.hget(
+                f"mapping-{req['branch']}-{req['version']}-{req['target']}",
+                req["profile"],
             )
 
+            if mapped_profile:
+                req["profile"] = mapped_profile.decode()
+            else:
+                return (
+                    {"detail": f"Unsupported profile: {req['profile']}", "status": 400},
+                    400,
+                )
+
     package_problems = validate_packages(req)
     if package_problems:
         return package_problems
@@ -251,15 +265,8 @@ def validate_request(req):
     return ({}, None)
 
 
-def return_job(job):
-    """Return job status message and code
-
-    The states vary if the image is currently build, failed or finished
-
-    Returns:
-        (dict, int): Status message and code
-    """
-    response = {}
+def return_job_v1(job):
+    response = job.get_meta()
     headers = {}
     if job.meta:
         response.update(job.meta)
@@ -284,6 +291,7 @@ def return_job(job):
                 "detail": "started",
             }
         )
+        headers = {"X-Imagebuilder-Status": response.get("imagebuilder_status", "init")}
 
     elif job.is_finished:
         response.update({"status": 200, "build_at": job.ended_at, **job.result})
@@ -295,26 +303,13 @@ def return_job(job):
     return response, response["status"], headers
 
 
+# legacy offering /api/overview
+@bp.route("/overview")
 def api_v1_overview():
     return jsonify(current_app.config["OVERVIEW"])
 
 
-def api_build_get(request_hash):
-    return api_v1_build_get(request_hash)
-
-
 def api_v1_build_get(request_hash):
-    """API call to get job information based on `request_hash`
-
-    This API call can be used for polling once the initial build request is
-    accepted. The request using POST returns the `request_hash` on success.
-
-    Args:
-        request_hash (str): Request hash to lookup
-
-    Retrns:
-        (dict, int): Status message and code
-    """
     job = get_queue().fetch_job(request_hash)
     if not job:
         return {
@@ -323,35 +318,95 @@ def api_v1_build_get(request_hash):
             "detail": "could not find provided request hash",
         }, 404
 
-    return return_job(job)
-
-
-def api_build_post():
-    return api_v1_build_post()
+    return return_job_v1(job)
 
 
 def api_v1_build_post():
-    """API call to request an image
-
-    An API request contains at least version and profile. The `packages` key
-    can contain a list of extra packages to install.
-
-    Below an example of the contents of the POSTed JSON data:
-
-        {
-            "profile": "tplink_tl-wdr4300-v1",
-            "packages": [
-                "luci"
-            ],
-            "version": "SNAPSHOT"
-        }
-
-    Retrns:
-        (dict, int): Status message and code
-    """
     req = request.get_json()
     current_app.logger.debug(f"req {req}")
+    request_hash = get_request_hash(req)
+    job = get_queue().fetch_job(request_hash)
+    response = {}
+    status = 200
+    result_ttl = "7d"
+    failure_ttl = "12h"
 
+    if job is None:
+        response, status = validate_request(req)
+        if response:
+            return response, status
+
+        req["store_path"] = current_app.config["STORE_PATH"]
+        req["upstream_url"] = current_app.config["UPSTREAM_URL"]
+        req["branch_data"] = current_app.config["BRANCHES"][req["branch"]]
+        req["request_hash"] = request_hash
+
+        job = get_queue().enqueue(
+            build,
+            req,
+            job_id=request_hash,
+            result_ttl=result_ttl,
+            failure_ttl=failure_ttl,
+            job_timeout="10m",
+        )
+
+    return return_job_v1(job)
+
+
+# legacy /api/build
+@bp.route("/branches")
+def api_branches():
+    return jsonify(list(current_app.config["OVERVIEW"]["branches"].values()))
+
+
+def return_job(job):
+    response = {}
+    if job.meta:
+        response.update(job.meta)
+
+    status = 500
+
+    if job.is_failed:
+        response["message"] = job.exc_info.strip().split("\n")[-1]
+
+    elif job.is_queued:
+        status = 202
+        response = {
+            "status": job.get_status(),
+            "queue_position": job.get_position() or 0,
+        }
+
+    elif job.is_started:
+        status = 202
+        response = {
+            "status": job.get_status(),
+        }
+
+    elif job.is_finished:
+        status = 200
+        response.update(job.result)
+        response["build_at"] = job.ended_at
+
+    response["enqueued_at"] = job.enqueued_at
+    response["request_hash"] = job.id
+
+    current_app.logger.debug(f"Response {response} with status {status}")
+    return response, status
+
+
+@bp.route("/build/<path:request_hash>", methods=["GET"])
+def api_build_get(request_hash):
+    job = get_queue().fetch_job(request_hash)
+    if not job:
+        return {"status": "not_found"}, 404
+
+    return return_job(job)
+
+
+@bp.route("/build", methods=["POST"])
+def api_build_post():
+    req = request.get_json()
+    current_app.logger.debug(f"req {req}")
     request_hash = get_request_hash(req)
     job = get_queue().fetch_job(request_hash)
     response = {}
@@ -367,10 +422,7 @@ def api_v1_build_post():
         req["store_path"] = current_app.config["STORE_PATH"]
         req["upstream_url"] = current_app.config["UPSTREAM_URL"]
         req["branch_data"] = current_app.config["BRANCHES"][req["branch"]]
-
-        if req["branch_data"].get("snapshot"):
-            result_ttl = "15m"
-            current_app.logger.info(f"Set snapshot request {request_hash} ttl to 15m")
+        req["request_hash"] = request_hash
 
         job = get_queue().enqueue(
             build,
